xquery version "3.1";

module namespace app="http://grammateus.ch/templates";

import module namespace templates="http://exist-db.org/xquery/templates" ;
import module namespace config="http://grammateus.ch/config" at "config.xqm";
import module namespace gramm="http://grammateus.ch/gramm" at "gramm.xql";
import module namespace URI="http://grammateus.ch/URI" at "uri.xql";
import module namespace console = "http://exist-db.org/xquery/console";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace functx = "http://www.functx.com";

(:: TEMPLATE page.html ::)
declare %templates:wrap function app:set-page-title($node as node(), $model as map(*)){
    let $url := request:get-url()
    let $page_title := 
        if (matches($url, "index")) then "Home"
        else if (matches($url, "about")) then "About"
        else if (matches($url, "overview") or matches($url, "concepts")) then "Introduction"
        else if (matches($url, "typology")) then "Typology"
        else if (matches($url, "description")) then "Description"
        else if (matches($url, "classification")) then "Classification"
        else if (matches($url, "taxonomy")) then "Taxonomy"
        else if (matches($url, "browse")) then "Browse"
        else if (matches($url, "papyri.html")) then "Papyri"
        else if (matches($url, "bibliography.html")) then "Bibliography"
        else if (matches($url, "show")) then xs:string(request:get-parameter("document", ""))
        else ()
    return $page_title
};

(:: BROWSE website contents - search result page ::)
declare
%templates:default("keyword", "")
%templates:wrap
function app:browse($node as node(), $model as map(*), $keyword as xs:string){
    let $results := if ($keyword eq "") 
                    then () 
                    else 
                        (: search both TEI and HTML files :)
                        let $collections := collection("/db/apps/grammateus/")//xhtml:p[ft:query(., $keyword)] |
                            collection("/db/apps/grammateus/")//tei:p[ft:query(., $keyword)] |
                            collection("/db/apps/grammateus/")//tei:head[ft:query(., $keyword)] |
                            collection("/db/apps/grammateus/")//tei:p[descendant::tei:ptr[matches(@target, $keyword)]] |
                            collection("/db/apps/grammateus/")//xhtml:h1[ft:query(., $keyword)] |
                            collection("/db/apps/grammateus/")//xhtml:h3[ft:query(., $keyword)] |
                            collection("/db/apps/grammateus/")//xhtml:li[ft:query(., $keyword)]
                        for $match in $collections
                            let $document := document-uri(root($match))
                            let $expanded := util:expand($match, "expand-xincludes=no")
                            return 
                                <hit>
                                    <match>{$expanded}</match>
                                    <document>{$document}</document>
                                </hit>
    return
        map{"results": $results,
            "term": $keyword
        }
};
declare function app:term($node as node(), $model as map(*)) {
    if ($model("term") eq "") 
    then
        <p><span class="badge remove-filters">No search term!</span></p>
    else 
        <p><span class="badge badge-secondary">Search term: {$model("term")}</span></p>
};
declare %templates:wrap function app:hits($node as node(), $model as map(*)) {
    let $hits := count($model("results")//exist:match)
    let $res := if ($hits > 1) then "Results" else "Result"
    return
        $hits || " " || $res
};
declare function app:results($node as node(), $model as map(*)) {
    (: the $hit/match is a node, where the xml is not transformed into HTML
        there is some styling missing e.g. for bibl entries or greek terms. :)
    for $hit in $model("results")
        let $expanded := $hit/match
        let $link := $hit/document
        (: group by page where hits appear :)
        group by $link
        return 
            gramm:result($expanded, $link)
};


(:: ABOUT page ::)
declare function app:team($node as node(), $model as map(*)){
    <div>
        <ul>
            {
                let $doc := doc(concat($config:data-root, '/authority_lists.xml'))
                for $member in $doc//tei:respStmt
                return
                    <li><a target="_blank" href="{$member//tei:ref/@target}">{$member/tei:name}</a>: {$member/tei:resp}</li>
            }
        </ul>
    </div>
};

declare function app:license($node as node(), $model as map(*)){
        <div>
            <a rel="license" href="http://creativecommons.org/licenses/by-nc/4.0/"><img alt="Creative Commons License" style="border-width:0" src="https://i.creativecommons.org/l/by-nc/4.0/88x31.png" /></a>
            <br/>
            This work is licensed under a <a rel="license" href="http://creativecommons.org/licenses/by-nc/4.0/">Creative Commons Attribution-NonCommercial 4.0 International License</a>.
        </div>
};

(:: INTRODUCTION pages - overview, concpets ::)
declare function app:overview($node as node(), $model as map(*)) {
    (: get xml file as doc :)
    let $doc := doc($URI:intro||"/intro_general.xml")
    return
        map {
            "document": $doc
        }
};
declare function app:concepts($node as node(), $model as map(*)) {
    (: get xml file as doc :)
    let $doc := doc($URI:intro||"/intro_concepts.xml")
    return
        map {
            "document": $doc
        }
};
declare function app:short-sidebar($node as node(), $model as map(*)) {
    <div class="col-md-3 sidebar-container">
        <div class="sidebar">
            <div class="sidebar-item level0">
                <span class="sidebar-link">{$model("document")//tei:body/tei:head}</span>
            </div>
            {
                let $doc := $model("document")
                for $section in $doc//tei:div[@type eq "section"]
                return
                    <div class="sidebar-item level1">
                        <a class="sidebar-link" href="#{$section/tei:head/@xml:id}">{$section/tei:head/string()}</a>
                        {
                            for $subsection in $section//tei:div[@type eq "subsection"]
                            return
                                <div class="sidebar-item level2">
                                    <a class="sidebar-link" href="#{$subsection/tei:head/@xml:id}">{$subsection/tei:head/string()}</a>
                                </div>
                        }
                    </div>
            }
        </div>
    </div>
};

(:: TYPOLOGY pages - typology, description, classification ::)
declare function app:description($node as node(), $model as map(*)) {
    let $document := xs:string(request:get-parameter("d", "typology.xml"))
    
    (: get xml file as doc :)
    let $doc := doc(replace(concat($URI:descr,"/",$document), '/exist/', '/db/'))
    
    return
        map {
            "filename": $document,
            "document": $doc,
            "taxonomy": doc($URI:taxonomy),
            "nav-title": "Descriptions",
            "nav-link": "description.html"
        }
};
declare function app:classification($node as node(), $model as map(*)) {
    let $document := xs:string(request:get-parameter("d", "typology.xml"))
    
    (: get xml file as doc :)
    let $doc := doc(replace(concat($URI:class,"/",$document), '/exist/', '/db/'))
    
    return
        map {
            "filename": $document,
            "document": $doc,
            "taxonomy": doc($URI:taxonomy),
            "nav-title": "Classification",
            "nav-link": "classification.html"
        }
};
declare function app:sidebar($node as node(), $model as map(*)) {
        <div class="col-md-3 sidebar-container">
            {gramm:sidebar($model)}
        </div>
};
declare function app:text($node as node(), $model as map(*)) {
    let $doc := $model("document")
    (: transform to html :)
    let $xsl := doc(concat($config:app-root, "/resources/xslt/typologyToHTML.xslt"))
    let $html := transform:transform($doc, $xsl, ())
    return
        <main class="col-md-7">
             {$html}   
        </main>
};
declare function app:typology-list($node as node(), $model as map(*)) {
    let $xml := doc($URI:taxonomy)
    let $xsl := doc("/db/apps/grammateus/resources/xslt/taxonomyToList.xslt")
    return
        transform:transform($xml, $xsl, ())
};


(:: BIBLIOGRAPHY page ::)
declare function app:bibliography($node as node(), $model as map(*)) {
    
    let $references := (collection($URI:intro)//tei:bibl/tei:ptr/@target/string(),
            collection($URI:descr)//tei:bibl/tei:ptr/@target/string(),
            collection($URI:class)//tei:bibl/tei:ptr/@target/string()
        )
    (: find distinct bibliographic references :)
    for $key in distinct-values($references)
        
        (: get the key :)
        let $k := $key => substring-after("gramm:")
    
        (: get the reference document from key :)
        let $doc := doc("/db/apps/grammateus/data/bibl/"||$k||".xml")
    
        (: count authors/editors for a proper display :)
        let $author := count($doc//tei:text//tei:author)
        let $editor := count($doc//tei:text//tei:editor)
    
        (: date for sorting :)
        let $date := $doc//tei:imprint/tei:date
    
        (: sorting variable :)
        (: order by first author, if not by first editor, or by key :)
        let $sort := if($doc//tei:text//tei:author) then ($doc//tei:text//tei:author[1]/tei:surname)
                else if($doc//tei:text//tei:editor) then ($doc//tei:text//tei:editor[1]/tei:surname)
                else $key
    
        (: first author/editor, and if several items by the same, then use the date :)
        order by $sort, $key, $date
    
    return
        <p><a class="anchor" id="{$k}"></a>
            <!-- author -->
           {(: reference with link :)
            if ($doc//tei:monogr//tei:ref/@target/string()) then
                <a href="{$doc//tei:monogr//tei:ref/@target/string()}">
                    {app:set-author($doc, $author, $editor)}
                </a>
            (: reference without link :)
            else app:set-author($doc, $author, $editor)
            }
            <!-- date -->
            <span>. {$date}.</span>
            
            <!-- title -->
            {   (: journal article :)
                if ($doc//tei:title[@level eq "j"])
                then app:journal-title($doc)
                (: book chapter :)
                else if ($doc//tei:analytic/tei:title[@level eq "a"] and $doc//tei:monogr/tei:title[@level eq "m"])
                then app:bookchapter-title($doc)
                (: monograph :)
                else app:book-title($doc)
            }
            
            <!-- links to online versions -->
            {if ($doc//tei:idno[@subtype eq "OA"]) then <span> Available Open Access: <a href="{$doc//tei:idno[@subtype eq "OA"]}">{$doc//tei:idno[@subtype eq "OA"]}</a></span>
            else if ($doc//tei:idno[@subtype eq "doc"]) then <span> Available Online: <a href="{$doc//tei:idno[@subtype eq "doc"]}">{$doc//tei:idno[@subtype eq "doc"]}</a></span>
            else ()}
        </p>
};

declare function app:set-author($doc, $author, $editor) {
    (:author:)
    if ($author = 1) then app:one-author($doc//tei:text//tei:author)
    else if ($author = 2) then app:two-author($doc//tei:text//tei:author)
    else if ($author gt 2) then app:multi-author($doc//tei:text//tei:author)
    
    (: no author/editor only :)
    else if ($editor = 1) then app:one-author($doc//tei:text//tei:editor)
    else if ($editor = 2) then app:two-author($doc//tei:text//tei:editor)
    else app:multi-author($doc//tei:text//tei:editor)
};

declare function app:one-author($author) {
    <span>{$author[1]/tei:surname}, {substring($author[1]/tei:forename, 1, 1)}</span>
};

declare function app:two-author($author) {
    <span>{$author[1]/tei:surname}, {substring($author[1]/tei:forename, 1, 1)}. &amp; {$author[2]/tei:surname}, {substring($author[2]/tei:forename, 1, 1)}</span>
};

declare function app:multi-author($author) {
    <span>{$author[1]/tei:surname}, {substring($author[1]/tei:forename, 1, 1)}.&#32;<i>et al</i></span>
};

declare function app:journal-title($doc) {
    <span>&#32;
           <!-- article title-->
            <span>"{$doc//tei:biblStruct//tei:title[@level eq "a"]/string()}"</span>
            <!-- journal title, short if possible -->
            <i>{
                if ($doc//tei:biblStruct//tei:title[@level eq "j" and @type eq "short"]) then 
                    <span>,&#32;{$doc//tei:biblStruct//tei:title[@level eq "j" and @type eq "short"]/string()}</span>
                else if ($doc//tei:biblStruct//tei:title[@level eq "j"]) 
                then <span>,&#32;{$doc//tei:biblStruct//tei:title[@level eq "j"]/string()}</span>
                else ()
            }</i>
            <!-- volume -->
            {if ($doc//tei:biblScope[@unit eq "vol"]) 
            then <span>&#32;({$doc//tei:biblScope[@unit eq "vol"]})</span> 
            else ()}
            <!-- pages -->
            {if ($doc//tei:biblScope[@unit eq "pp"]) 
            then <span>,&#32;{$doc//tei:biblScope[@unit eq "pp"]}</span> 
            else ()}
            <span>.</span>
    </span>
};

declare function app:book-title($doc) {
    <span>&#32;<i>{$doc//tei:biblStruct//tei:title[@level eq "m"][@type eq "main"]/string()}</i>.</span>
};

declare function app:bookchapter-title($doc) {
    <span>&#32;
        "{$doc//tei:biblStruct//tei:title[@level eq "a"]/string()}".&#32;In&#32;
        <i>{$doc//tei:biblStruct//tei:title[@level eq "m"]/string()}</i>,&#32;
        edited by {app:book-editor($doc//tei:biblStruct//tei:monogr)},&#32;
        pp. {$doc//tei:biblScope[@unit eq "pp"]}.
    </span>
};

declare function app:book-editor($monogr) {
    (: 1 editor :)
    if (count($monogr//tei:editor) = 1) 
    then $monogr//tei:editor/tei:forename || " " || $monogr//tei:editor/tei:surname
    (: 2 editors :)
    else if (count($monogr//tei:editor) = 2)
    then $monogr//tei:editor[1]/tei:forename || " " || $monogr//tei:editor[1]/tei:surname || " and " ||
    $monogr//tei:editor[2]/tei:forename || " " || $monogr//tei:editor[2]/tei:surename
    (: 3 and more editors :)
    else
        <span>{$monogr//tei:editor[1]/tei:forename || " " || $monogr//tei:editor[1]/tei:surname} <i>et al.</i></span>
};
